part of tecfy_basic_package;

abstract class BaseController {
  bool busy = false;

  void dispose() {}

  State? state;
  late BuildContext _context;
  void Function(VoidCallback)? setStateFunction;
  BuildContext get context {
    return state?.context ?? _context;
  }

  BaseController(this.state, {buildContext})
      : assert(state != null || buildContext != null) {
    _context = buildContext;
  }

  navigateTo(creator,
      [bool replacment = false,
      AnimationTypes animationType = AnimationTypes.slideLeft]) {
    return AppNavigator.navigateTo(context, creator,
        replacment: replacment, animationType: animationType);
  }

  setState() {
    if (setStateFunction != null)
      setStateFunction?.call(() {});
    else if (state != null && (state?.mounted ?? false)) state?.setState(() {});
  }

  Future<dynamic> showMessage(String title,
      {String heroTag = 'submit_button',
      MessageType type = MessageType.none,
      String? message,
      String? imageAssert}) async {
    if (busy) return;
    busy = true;
    await AppNavigator.showMessage(context, title,
        heroTag: heroTag,
        type: type,
        message: message,
        imageAssert: imageAssert);
    busy = false;
  }

  Future<bool> confirmMessage(String title,
      {String? message, String? imageAssert}) async {
    if (busy) return false;
    busy = true;
    var result = await AppNavigator.confirmMessage(context, title,
        message: message, imageAssert: imageAssert);
    busy = false;
    return result;
  }

  setLoading() async {
    if (busy) return;
    busy = true;
    await AppNavigator.showLoading(context);
    busy = false;
  }

  setResume() {
    if (!busy) return;
    busy = false;
    Navigator.of(context, rootNavigator: true).pop();
  }

  Future<void> hapticFeedback({bool heavy = true}) {
    return AppNavigator.hapticFeedback(heavy: heavy);
  }
}
